@@ -14,6 +14,7 @@
 
 import collections
 import filecmp
+import io
 import json
 import logging
 import os
@@ -21,21 +22,20 @@ import re
 import shutil
 import sys
 import tempfile
+from unittest import mock
 
 import fixtures
 from keystoneclient import fixture
 from keystoneclient.fixture import v2 as ks_v2_fixture
 from keystoneclient.fixture import v3 as ks_v3_fixture
-import mock
 from oslo_log import handlers
 from oslo_log import log
 import requests_mock
-import six
 from testtools import matchers
 
+from muranoclient.apiclient import exceptions
 from muranoclient.common import exceptions as common_exceptions
 from muranoclient.common import utils
-from muranoclient.openstack.common.apiclient import exceptions
 import muranoclient.shell
 from muranoclient.tests.unit import base
 from muranoclient.tests.unit import test_utils
@@ -117,8 +117,8 @@ class ShellCommandTest(ShellTest):
         orig = sys.stdout
         orig_stderr = sys.stderr
         try:
-            sys.stdout = six.StringIO()
-            sys.stderr = six.StringIO()
+            sys.stdout = io.StringIO()
+            sys.stderr = io.StringIO()
             _shell = muranoclient.shell.MuranoShell()
             _shell.main(argstr.split())
         except SystemExit:
@@ -153,9 +153,9 @@ class ShellCommandTest(ShellTest):
 
     def test_help(self):
         required = [
-            '.*?^usage: murano',
-            '.*?^\s+package-create\s+Create an application package.',
-            '.*?^See "murano help COMMAND" for help on a specific command',
+            r'.*?^usage: murano',
+            r'.*?^\s+package-create\s+Create an application package.',
+            r'.*?^See "murano help COMMAND" for help on a specific command',
         ]
         stdout, stderr = self.shell('help')
         for r in required:
@@ -174,9 +174,9 @@ class ShellCommandTest(ShellTest):
 
     def test_help_no_options(self):
         required = [
-            '.*?^usage: murano',
-            '.*?^\s+package-create\s+Create an application package',
-            '.*?^See "murano help COMMAND" for help on a specific command',
+            r'.*?^usage: murano',
+            r'.*?^\s+package-create\s+Create an application package',
+            r'.*?^See "murano help COMMAND" for help on a specific command',
         ]
         stdout, stderr = self.shell('')
         for r in required:
@@ -236,7 +236,6 @@ class ShellCommandTest(ShellTest):
         self.shell('package-list')
         self.client.packages.filter.assert_called_once_with(
             include_disabled=False,
-            limit=20,
             owned=False)
 
     @mock.patch('muranoclient.v1.packages.PackageManager')
@@ -252,6 +251,19 @@ class ShellCommandTest(ShellTest):
             limit=10,
             owned=False)
 
+    @mock.patch('muranoclient.v1.packages.PackageManager')
+    @requests_mock.mock()
+    def test_package_list_with_marker(self, mock_package_manager, m_requests):
+        self.client.packages = mock_package_manager()
+        self.make_env()
+        self.register_keystone_discovery_fixture(m_requests)
+        self.register_keystone_token_fixture(m_requests)
+        self.shell('package-list --marker 12345')
+        self.client.packages.filter.assert_called_once_with(
+            include_disabled=False,
+            marker='12345',
+            owned=False)
+
     @mock.patch('muranoclient.v1.packages.PackageManager')
     @requests_mock.mock()
     def test_package_list_with_name(self, mock_package_manager, m_requests):
@@ -263,8 +275,7 @@ class ShellCommandTest(ShellTest):
         self.client.packages.filter.assert_called_once_with(
             name='mysql',
             include_disabled=False,
-            owned=False,
-            limit=20)
+            owned=False)
 
     @mock.patch('muranoclient.v1.packages.PackageManager')
     @requests_mock.mock()
@@ -277,8 +288,7 @@ class ShellCommandTest(ShellTest):
         self.client.packages.filter.assert_called_once_with(
             fqn='mysql',
             include_disabled=False,
-            owned=False,
-            limit=20)
+            owned=False)
 
     @mock.patch('muranoclient.v1.packages.PackageManager')
     @requests_mock.mock()
@@ -398,11 +408,15 @@ class ShellCommandTest(ShellTest):
         self.register_keystone_token_fixture(m_requests)
 
         self.shell('environment-list')
-        self.client.environments.list.assert_called_once_with(False)
+        self.client.environments.list.assert_called_once_with(False, None)
 
         self.client.environments.list.reset_mock()
         self.shell('environment-list --all-tenants')
-        self.client.environments.list.assert_called_once_with(True)
+        self.client.environments.list.assert_called_once_with(True, None)
+
+        self.client.environments.list.reset_mock()
+        self.shell('environment-list --tenant ABC')
+        self.client.environments.list.assert_called_once_with(False, 'ABC')
 
     @mock.patch('muranoclient.v1.environments.EnvironmentManager')
     @requests_mock.mock()
@@ -459,6 +473,61 @@ class ShellCommandTest(ShellTest):
         self.client.environments.find.assert_called_once_with(
             name='env-id-or-name')
 
+    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
+    @requests_mock.mock()
+    def test_environment_model_show_basic(self, mock_manager, m_requests):
+        self.client.environments = mock_manager()
+        self.make_env()
+        self.register_keystone_discovery_fixture(m_requests)
+        self.register_keystone_token_fixture(m_requests)
+        self.shell('environment-model-show env-id')
+        self.client.environments.get_model.assert_called_once_with(
+            'env-id', '/', None)
+
+    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
+    @requests_mock.mock()
+    def test_environment_model_show_full(self, mock_manager, m_requests):
+        self.client.environments = mock_manager()
+        self.make_env()
+        self.register_keystone_discovery_fixture(m_requests)
+        self.register_keystone_token_fixture(m_requests)
+        self.shell('environment-model-show env-id '
+                   '--path /path '
+                   '--session-id sess-id')
+        self.client.environments.get_model.assert_called_once_with(
+            'env-id', '/path', 'sess-id')
+
+    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
+    @requests_mock.mock()
+    def test_environment_model_show_path_with_q(self, mock_manager,
+                                                m_requests):
+        self.client.environments = mock_manager()
+        self.make_env()
+        self.register_keystone_discovery_fixture(m_requests)
+        self.register_keystone_token_fixture(m_requests)
+        self.shell('environment-model-show env-id '
+                   '--path /?/path')
+        self.client.environments.get_model.assert_called_once_with(
+            'env-id', '/%3F/path', None)
+
+    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
+    @requests_mock.mock()
+    def test_environment_model_edit(self, mock_manager, m_requests):
+        self.client.environments = mock_manager()
+        self.make_env()
+        self.register_keystone_discovery_fixture(m_requests)
+        self.register_keystone_token_fixture(m_requests)
+        self.client.environments.get_model.return_value = {'name': "foo"}
+        temp_file = tempfile.NamedTemporaryFile(prefix="murano-test", mode='w')
+        patch = [{'op': 'replace', 'path': '/name', 'value': 'dummy'}]
+        json.dump(patch, temp_file)
+        temp_file.file.flush()
+
+        self.shell('environment-model-edit env-id {0} '
+                   '--session-id sess-id'.format(temp_file.name))
+        self.client.environments.update_model.assert_called_once_with(
+            'env-id', patch, 'sess-id')
+
     @mock.patch('muranoclient.v1.environments.EnvironmentManager')
     @mock.patch('muranoclient.v1.sessions.SessionManager')
     @requests_mock.mock()
@@ -706,7 +775,20 @@ class ShellCommandTest(ShellTest):
         self.register_keystone_token_fixture(m_requests)
         self.shell('env-template-create-env env-id env-name')
         self.client.env_templates.create_env.\
-            assert_called_once_with('env-id', 'env-name')
+            assert_called_once_with('env-id', {"name": 'env-name'})
+
+    @mock.patch('muranoclient.v1.templates.EnvTemplateManager')
+    @requests_mock.mock()
+    def test_env_template_create_env_with_region(self, mock_manager,
+                                                 m_requests):
+        self.client.env_templates = mock_manager()
+        self.make_env()
+        self.register_keystone_discovery_fixture(m_requests)
+        self.register_keystone_token_fixture(m_requests)
+        self.shell('env-template-create-env env-id env-name --region Region')
+        self.client.env_templates.create_env.\
+            assert_called_once_with('env-id', {"name": 'env-name',
+                                               "region": 'Region'})
 
     @mock.patch('muranoclient.v1.templates.EnvTemplateManager')
     @requests_mock.mock()
@@ -734,6 +816,44 @@ class ShellCommandTest(ShellTest):
             name='env-id-or-name')
         self.assertEqual(1, self.client.deployments.list.call_count)
 
+    @mock.patch('muranoclient.v1.deployments.DeploymentManager')
+    @requests_mock.mock()
+    def test_deployments_list_all_environments(self, mock_deployment_manager,
+                                               m_requests):
+        self.client.deployments = mock_deployment_manager()
+        self.make_env()
+        self.register_keystone_discovery_fixture(m_requests)
+        self.register_keystone_token_fixture(m_requests)
+        self.shell('deployment-list --all-environments')
+        self.client.deployments.list.assert_called_once_with(
+            None, True)
+
+    @mock.patch('muranoclient.v1.deployments.DeploymentManager')
+    @requests_mock.mock()
+    def test_deployments_list_negative(self, mock_deployment_manager,
+                                       m_requests):
+        self.client.deployments = mock_deployment_manager()
+        self.make_env()
+        self.register_keystone_discovery_fixture(m_requests)
+        self.register_keystone_token_fixture(m_requests)
+        e = self.assertRaises(
+            exceptions.CommandError,
+            self.shell,
+            'deployment-list env-id --all-environments')
+        self.assertIn(
+            'Environment ID and all-environments flag cannot both be set.',
+            e.__str__())
+        self.assertFalse(self.client.deployments.list.called)
+
+        e = self.assertRaises(
+            exceptions.CommandError,
+            self.shell,
+            'deployment-list')
+        self.assertIn(
+            'Either environment ID or all-environments flag must be set.',
+            e.__str__())
+        self.assertFalse(self.client.deployments.list.called)
+
     @mock.patch('muranoclient.v1.services.ServiceManager')
     @mock.patch('muranoclient.v1.environments.EnvironmentManager')
     @requests_mock.mock()
@@ -878,7 +998,7 @@ class ShellCommandTest(ShellTest):
         ex = self.assertRaises(exceptions.CommandError, self.shell,
                                'category-delete category-id')
         expected = 'Unable to find and delete any of the specified categories.'
-        self.assertEqual(expected, six.text_type(ex))
+        self.assertEqual(expected, str(ex))
 
 
 class ShellPackagesOperations(ShellCommandTest):
@@ -958,7 +1078,7 @@ class ShellPackagesOperations(ShellCommandTest):
             v1_shell.do_package_import(self.client, args)
             return f.name
 
-    @mock.patch('six.moves.input')
+    @mock.patch('builtins.input')
     @mock.patch('muranoclient.common.utils.Package.from_file')
     def test_package_import_conflict_skip(self, from_file, raw_input_mock):
 
@@ -973,7 +1093,7 @@ class ShellPackagesOperations(ShellCommandTest):
             'is_public': False,
         }, {name: mock.ANY},)
 
-    @mock.patch('six.moves.input')
+    @mock.patch('builtins.input')
     @mock.patch('muranoclient.common.utils.Package.from_file')
     def test_package_import_conflict_skip_ea(self, from_file, raw_input_mock):
 
@@ -990,7 +1110,7 @@ class ShellPackagesOperations(ShellCommandTest):
         }, {name: mock.ANY},)
         self.assertFalse(raw_input_mock.called)
 
-    @mock.patch('six.moves.input')
+    @mock.patch('builtins.input')
     @mock.patch('muranoclient.common.utils.Package.from_file')
     def test_package_import_conflict_abort(self, from_file, raw_input_mock):
 
@@ -1005,7 +1125,7 @@ class ShellPackagesOperations(ShellCommandTest):
             'is_public': False,
         }, mock.ANY,)
 
-    @mock.patch('six.moves.input')
+    @mock.patch('builtins.input')
     @mock.patch('muranoclient.common.utils.Package.from_file')
     def test_package_import_conflict_abort_ea(self,
                                               from_file, raw_input_mock):
@@ -1023,7 +1143,7 @@ class ShellPackagesOperations(ShellCommandTest):
         }, mock.ANY,)
         self.assertFalse(raw_input_mock.called)
 
-    @mock.patch('six.moves.input')
+    @mock.patch('builtins.input')
     @mock.patch('muranoclient.common.utils.Package.from_file')
     def test_package_import_conflict_update(self, from_file, raw_input_mock):
 
@@ -1044,7 +1164,7 @@ class ShellPackagesOperations(ShellCommandTest):
         )
         self.assertEqual(2, self.client.packages.create.call_count)
 
-    @mock.patch('six.moves.input')
+    @mock.patch('builtins.input')
     @mock.patch('muranoclient.common.utils.Package.from_file')
     def test_package_import_conflict_update_ea(self,
                                                from_file, raw_input_mock):
@@ -1128,13 +1248,13 @@ class ShellPackagesOperations(ShellCommandTest):
 
     @mock.patch('muranoclient.common.utils.Package.from_file')
     def test_package_import_conflict_dep_update_ea(self, from_file):
-        self._test_conflict_dep(
-            self.client.packages,
-            from_file,
-            dep_exists_action='u',
-        )
+        self.assertRaises(SystemExit, self._test_conflict_dep,
+                          self.client.packages,
+                          from_file,
+                          dep_exists_action='u',
+                          )
 
-        self.assertEqual(True, self.client.packages.delete.called)
+        self.assertTrue(self.client.packages.delete.called)
 
         self.client.packages.create.assert_has_calls(
             [
@@ -1144,8 +1264,8 @@ class ShellPackagesOperations(ShellCommandTest):
             ], any_order=True,
         )
 
-        self.assertEqual(True, self.client.packages.create.call_count > 2)
-        self.assertEqual(True, self.client.packages.create.call_count < 5)
+        self.assertGreater(self.client.packages.create.call_count, 2)
+        self.assertLess(self.client.packages.create.call_count, 5)
 
     @mock.patch('muranoclient.common.utils.Package.from_file')
     def test_package_import_no_categories(self, from_file):
@@ -1247,13 +1367,13 @@ class ShellPackagesOperations(ShellCommandTest):
         m.get(TestArgs.murano_repo_url + '/apps/first_app.zip', body=pkg1)
         m.get(TestArgs.murano_repo_url + '/apps/second_app.1.0.zip',
               body=pkg2)
-        s = six.StringIO()
+        s = io.StringIO()
         bundle_contents = {'Packages': [
             {'Name': 'first_app'},
             {'Name': 'second_app', 'Version': '1.0'}
         ]}
         json.dump(bundle_contents, s)
-        s = six.BytesIO(s.getvalue().encode('ascii'))
+        s = io.BytesIO(s.getvalue().encode('ascii'))
 
         m.get(TestArgs.murano_repo_url + '/bundles/test_bundle.bundle',
               body=s)
@@ -1285,13 +1405,13 @@ class ShellPackagesOperations(ShellCommandTest):
         m.get(TestArgs.murano_repo_url + '/apps/second_app.1.0.zip',
               body=pkg2)
 
-        s = six.StringIO()
+        s = io.StringIO()
         # bundle only contains 1st package
         bundle_contents = {'Packages': [
             {'Name': 'first_app'},
         ]}
         json.dump(bundle_contents, s)
-        s = six.BytesIO(s.getvalue().encode('ascii'))
+        s = io.BytesIO(s.getvalue().encode('ascii'))
 
         m.get(TestArgs.murano_repo_url + '/bundles/test_bundle.bundle',
               body=s)
@@ -1317,13 +1437,13 @@ class ShellPackagesOperations(ShellCommandTest):
         m.get(TestArgs.murano_repo_url + '/apps/first_app.zip', body=pkg1)
         m.get(TestArgs.murano_repo_url + '/apps/second_app.1.0.zip',
               body=pkg2)
-        s = six.StringIO()
+        s = io.StringIO()
         bundle_contents = {'Packages': [
             {'Name': 'first_app'},
             {'Name': 'second_app', 'Version': '1.0'}
         ]}
         json.dump(bundle_contents, s)
-        s = six.BytesIO(s.getvalue().encode('ascii'))
+        s = io.BytesIO(s.getvalue().encode('ascii'))
 
         url = 'http://127.0.0.2/test_bundle.bundle'
         m.get(url, body=s)
@@ -1416,12 +1536,12 @@ class ShellPackagesOperations(ShellCommandTest):
 
         m.get(TestArgs.murano_repo_url + '/apps/test_app.zip', body=pkg)
 
-        s = six.StringIO()
+        s = io.StringIO()
         expected_bundle = {'Packages': [
             {'Name': 'test_app'},
         ]}
         json.dump(expected_bundle, s)
-        s = six.BytesIO(s.getvalue().encode('ascii'))
+        s = io.BytesIO(s.getvalue().encode('ascii'))
 
         m.get(TestArgs.murano_repo_url + '/bundles/test_bundle.bundle',
               body=s)
